part of mvu_flutter;

/// Root [Program] widget.
///
/// Works essentially as Elm's version, but model and
/// commands arguments are split to reduce boilerplate because of Dart's
/// absence of algebraic data types.
class Program<ModelType extends Model, MsgType extends Msg<ModelType>>
    extends StatefulWidget {
  final ModelType model;
  final InitialView<ModelType, MsgType> view;
  final Update<ModelType, MsgType> update;
  final List<Cmd> commands;
  final Subscriptions? subscriptions;

  Program(
      {required this.model,
      required this.view,
      required this.update,
      this.commands = const [],
      this.subscriptions});

  Future<void> _dispatch(MsgType message) async =>
      dispatchGlobal<ModelType, MsgType>(message);

  _ProgramState<ModelType, MsgType> createState() =>
      _ProgramState<ModelType, MsgType>();
}

class _ProgramState<ModelType extends Model, MsgType extends Msg<ModelType>>
    extends State<Program> {
  void _disposeProgram() => _Runtime.removeManager<ModelType, MsgType>();

  List<StreamSubscription>? _subscriptions;

  @override
  void initState() {
    super.initState();
    _Runtime.addManager<ModelType, MsgType>(
      _Manager<ModelType, MsgType>(
          model: widget.model as ModelType,
          updater: widget.update as Update<ModelType, MsgType>,
          commands: widget.commands),
    );
    if (widget.subscriptions != null) {
      _subscriptions = widget.subscriptions!(widget._dispatch);
    }
  }

  @override
  void dispose() async {
    _disposeProgram();
    super.dispose();
    if (_subscriptions != null) {
      for (var subscription in _subscriptions!) {
        await subscription.cancel();
      }
    }
  }

  Widget build(BuildContext context) => ValueListenableBuilder(
        valueListenable: _Runtime.getManager<ModelType, MsgType>()!.model,
        builder: (context, _v, _c) => widget.view(),
      );
}
