part of mvu_flutter;

/// [Program] [Manager]. Only mutable part that is not accessed by package's user.
/// 
/// Stores model and updater, used by [dispatch]/[dispatchGlobal] to receive
/// [Msg]s, update model accordingly and execute commands.
class _Manager<ModelType extends Model, MsgType extends Msg<ModelType>> {
  final ValueNotifier<ModelType> model;
  final Update<ModelType, MsgType> updater;

  Future<void> update(MsgType message) async {
    final producedState = updater.update(message, model.value);
    model.value = producedState.model;
    await _executeCommands(producedState.commands);
  }

  Future<void> _executeCommands(List<Cmd> commands) async {
    for (var command in commands) {
      await command(update as Dispatch);
    }
  }

  _Manager(
      {required ModelType model,
      required this.updater,
      required List<Cmd> commands})
      : model = ValueNotifier(model) {
    _executeCommands(commands);
  }
}
