part of mvu_flutter;

/// Static [Manager]s' store, used to access specified Manager by its type.
///
/// [getManager] call will access the last added Manager of given type if
/// several are being used.
abstract class _Runtime {
  static ListQueue<_Manager> _managers = ListQueue();

  static void
      addManager<ModelType extends Model, MsgType extends Msg<ModelType>>(
              _Manager<ModelType, MsgType> manager) =>
          _managers.addLast(manager);

  static _Manager<ModelType, MsgType>?
      getManager<ModelType extends Model, MsgType extends Msg<ModelType>>() {
    final foundManagers = _managers.whereType<_Manager<ModelType, MsgType>>();
    if (foundManagers.isNotEmpty) return foundManagers.last;
  }

  static void removeManager<ModelType extends Model,
          MsgType extends Msg<ModelType>>() =>
      _managers.remove(getManager<ModelType, MsgType>());

  static void isProgramExists<ModelType extends Model,
      MsgType extends Msg<ModelType>>() {
    if (getManager<ModelType, MsgType>() == null) {
      throw Exception("There is no corresponding Program of model $ModelType");
    }
  }
}
